import { LRUCache } from "lru-cache";
import { auth } from "./auth.js";
import { getLog } from "./helpers/get-log.js";
import { getProbotOctokitWithDefaults } from "./octokit/get-probot-octokit-with-defaults.js";
import { getWebhooks } from "./octokit/get-webhooks.js";
import { ProbotOctokit } from "./octokit/probot-octokit.js";
import { VERSION } from "./version.js";
import { defaultWebhooksPath } from "./server/server.js";
import { rebindLog } from "./helpers/rebind-log.js";
export class Probot {
    static version = VERSION;
    static defaults(defaults) {
        const ProbotWithDefaults = class extends this {
            constructor(...args) {
                const options = args[0] || {};
                super(Object.assign({}, defaults, options));
            }
        };
        return ProbotWithDefaults;
    }
    webhooks;
    webhookPath;
    log;
    version;
    on;
    onAny;
    onError;
    auth;
    state;
    constructor(options = {}) {
        options.secret = options.secret || "development";
        let level = options.logLevel;
        const logMessageKey = options.logMessageKey;
        this.log = options.log
            ? rebindLog(options.log)
            : getLog({ level, logMessageKey });
        // TODO: support redis backend for access token cache if `options.redisConfig`
        const cache = new LRUCache({
            // cache max. 15000 tokens, that will use less than 10mb memory
            max: 15000,
            // Cache for 1 minute less than GitHub expiry
            ttl: 1000 * 60 * 59,
        });
        const Octokit = getProbotOctokitWithDefaults({
            githubToken: options.githubToken,
            Octokit: options.Octokit || ProbotOctokit,
            appId: Number(options.appId),
            privateKey: options.privateKey,
            cache,
            log: rebindLog(this.log),
            redisConfig: options.redisConfig,
            baseUrl: options.baseUrl,
        });
        const octokitLogger = rebindLog(this.log.child({ name: "octokit" }));
        const octokit = new Octokit({
            request: options.request,
            log: {
                debug: octokitLogger.debug.bind(octokitLogger),
                info: octokitLogger.info.bind(octokitLogger),
                warn: octokitLogger.warn.bind(octokitLogger),
                error: octokitLogger.error.bind(octokitLogger),
            },
        });
        this.state = {
            cache,
            githubToken: options.githubToken,
            log: rebindLog(this.log),
            Octokit,
            octokit,
            webhooks: {
                secret: options.secret,
            },
            appId: Number(options.appId),
            privateKey: options.privateKey,
            host: options.host,
            port: options.port,
            webhookPath: options.webhookPath || defaultWebhooksPath,
            request: options.request,
        };
        this.auth = auth.bind(null, this.state);
        this.webhooks = getWebhooks(this.state);
        this.webhookPath = this.state.webhookPath;
        this.on = this.webhooks.on;
        this.onAny = this.webhooks.onAny;
        this.onError = this.webhooks.onError;
        this.version = VERSION;
    }
    receive(event) {
        this.log.debug({ event }, "Webhook received");
        return this.webhooks.receive(event);
    }
    async load(appFn, options = {}) {
        if (Array.isArray(appFn)) {
            for (const fn of appFn) {
                await this.load(fn);
            }
            return;
        }
        return appFn(this, options);
    }
}
//# sourceMappingURL=probot.js.map