import { join, dirname } from "node:path";
import { fileURLToPath } from "node:url";
import express, { Router } from "express";
import { createNodeMiddleware as createWebhooksMiddleware } from "@octokit/webhooks";
import { getLoggingMiddleware } from "./logging-middleware.js";
import { createWebhookProxy } from "../helpers/webhook-proxy.js";
import { VERSION } from "../version.js";
import { rebindLog } from "../helpers/rebind-log.js";
// the default path as defined in @octokit/webhooks
export const defaultWebhooksPath = "/api/github/webhooks";
const __dirname = dirname(fileURLToPath(import.meta.url));
export class Server {
    static version = VERSION;
    expressApp;
    log;
    version = VERSION;
    probotApp;
    state;
    constructor(options = {}) {
        this.expressApp = express();
        this.probotApp = new options.Probot({
            request: options.request,
        });
        this.log = options.log
            ? rebindLog(options.log)
            : rebindLog(this.probotApp.log.child({ name: "server" }));
        this.state = {
            cwd: options.cwd || process.cwd(),
            port: options.port,
            host: options.host,
            webhookPath: options.webhookPath || defaultWebhooksPath,
            webhookProxy: options.webhookProxy,
        };
        this.expressApp.use(getLoggingMiddleware(this.log, options.loggingOptions));
        this.expressApp.use("/probot/static/", express.static(join(__dirname, "..", "..", "static")));
        this.expressApp.use(createWebhooksMiddleware(this.probotApp.webhooks, {
            path: this.state.webhookPath,
        }));
        this.expressApp.get("/ping", (_req, res) => res.end("PONG"));
    }
    async load(appFn) {
        await appFn(this.probotApp, {
            cwd: this.state.cwd,
            getRouter: (path) => this.router(path),
        });
    }
    async start() {
        this.log.info(`Running Probot v${this.version} (Node.js: ${process.version})`);
        const port = this.state.port || 3000;
        const { host, webhookPath, webhookProxy } = this.state;
        const printableHost = host ?? "localhost";
        this.state.httpServer = await new Promise((resolve, reject) => {
            const server = this.expressApp.listen(port, ...(host ? [host] : []), async () => {
                if (webhookProxy) {
                    this.state.eventSource = await createWebhookProxy({
                        logger: this.log,
                        path: webhookPath,
                        port: port,
                        url: webhookProxy,
                    });
                }
                this.log.info(`Listening on http://${printableHost}:${port}`);
                resolve(server);
            });
            server.on("error", (error) => {
                if (error.code === "EADDRINUSE") {
                    error = Object.assign(error, {
                        message: `Port ${port} is already in use. You can define the PORT environment variable to use a different port.`,
                    });
                }
                this.log.error(error);
                reject(error);
            });
        });
        return this.state.httpServer;
    }
    async stop() {
        if (this.state.eventSource)
            this.state.eventSource.close();
        if (!this.state.httpServer)
            return;
        const server = this.state.httpServer;
        return new Promise((resolve) => server.close(resolve));
    }
    router(path = "/") {
        const newRouter = Router();
        this.expressApp.use(path, newRouter);
        return newRouter;
    }
}
//# sourceMappingURL=server.js.map